@@ -3,7 +3,7 @@ var topics =
     [ "Types", "group__types_group.html", "group__types_group" ],
     [ "Callbacks", "group__callbacks_group.html", "group__callbacks_group" ],
     [ "Functions", "group__func_group.html", "group__func_group" ],
-    [ "- Microservices", "group__micro_group.html", "group__micro_group" ],
+    [ "Microservices", "group__micro_group.html", "group__micro_group" ],
     [ "Wildcards", "group__wildcards_group.html", null ],
     [ "Environment Variables", "group__env_variables_group.html", null ],
     [ "Libevent Adapter", "group__libevent_functions.html", "group__libevent_functions" ],
