@@ -1,4 +1,4 @@
-// Copyright 2015-2021 The NATS Authors
+// Copyright 2015-2025 The NATS Authors
 // Licensed under the Apache License, Version 2.0 (the "License");
 // you may not use this file except in compliance with the License.
 // You may obtain a copy of the License at
@@ -134,6 +134,15 @@ natsSock_ShuffleIPs(natsSockCtx *ctx, struct addrinfo **tmp, int tmpSize, struct
 
 #define MAX_HOST_NAME   (256)
 
+static void
+_resetDeadline(natsSockCtx* ctx, int64_t start, int64_t totalTimeout)
+{
+    int64_t used = nats_Now() - start;
+    int64_t left = totalTimeout - used;
+
+    natsDeadline_Init(&(ctx->writeDeadline), (left > 0 ? left : 0));
+}
+
 natsStatus
 natsSock_ConnectTcp(natsSockCtx *ctx, const char *phost, int port)
 {
@@ -170,12 +179,25 @@ natsSock_ConnectTcp(natsSockCtx *ctx, const char *phost, int port)
 
     snprintf(sport, sizeof(sport), "%d", port);
 
-    if ((ctx->orderIP == 46) || (ctx->orderIP == 64))
-        max = 2;
-
     start = nats_Now();
 
-    for (i=0; i<max; i++)
+    // Call the proxy connect callback if provided
+    if (ctx->proxyConnectCb != NULL)
+    {
+        // Invoke the proxy connect callback.
+        s = ctx->proxyConnectCb(&ctx->fd, host, port, ctx->proxyConnectClosure);
+
+        // If there was a deadline, reset the deadline with whatever is left.
+        if (totalTimeout > 0)
+            _resetDeadline(ctx, start, totalTimeout);
+
+        return NATS_UPDATE_ERR_STACK(s);
+    }
+
+    if ((ctx->orderIP == 46) || (ctx->orderIP == 64))
+	    max = 2;
+
+    for (i = 0; i < max; i++)
     {
         struct addrinfo hints;
         struct addrinfo *servinfo = NULL;
@@ -306,12 +328,7 @@ natsSock_ConnectTcp(natsSockCtx *ctx, const char *phost, int port)
 
     // If there was a deadline, reset the deadline with whatever is left.
     if (totalTimeout > 0)
-    {
-        int64_t used = nats_Now() - start;
-        int64_t left = totalTimeout - used;
-
-        natsDeadline_Init(&(ctx->writeDeadline), (left > 0 ? left : 0));
-    }
+        _resetDeadline(ctx, start, totalTimeout);
 
     return NATS_UPDATE_ERR_STACK(s);
 }
@@ -408,22 +425,17 @@ natsSock_Read(natsSockCtx *ctx, char *buffer, size_t maxBufferSize, int *n)
     {
 #if defined(NATS_HAS_TLS)
         if (ctx->ssl != NULL)
+        {
+            int sslErr = 0;
+
+            natsMutex_Lock(ctx->sslMu);
             readBytes = SSL_read(ctx->ssl, buffer, (int) maxBufferSize);
-        else
-#endif
-            readBytes = recv(ctx->fd, buffer, (natsRecvLen) maxBufferSize, 0);
+            if (readBytes < 0)
+                sslErr = SSL_get_error(ctx->ssl, readBytes);
+            natsMutex_Unlock(ctx->sslMu);
 
-        if (readBytes == 0)
-        {
-            return nats_setDefaultError(NATS_CONNECTION_CLOSED);
-        }
-        else if (readBytes < 0)
-        {
-#if defined(NATS_HAS_TLS)
-            if (ctx->ssl != NULL)
+            if (sslErr != 0)
             {
-                int sslErr = SSL_get_error(ctx->ssl, readBytes);
-
                 if (sslErr == SSL_ERROR_ZERO_RETURN)
                     return nats_setDefaultError(NATS_CONNECTION_CLOSED);
 
@@ -440,8 +452,17 @@ natsSock_Read(natsSockCtx *ctx, char *buffer, size_t maxBufferSize, int *n)
                     continue;
                 }
             }
+        }
+        else
 #endif
+            readBytes = recv(ctx->fd, buffer, (natsRecvLen) maxBufferSize, 0);
 
+        if (readBytes == 0)
+        {
+            return nats_setDefaultError(NATS_CONNECTION_CLOSED);
+        }
+        else if (readBytes < 0)
+        {
             if (NATS_SOCK_GET_ERROR != NATS_SOCK_WOULD_BLOCK)
             {
 #if defined(NATS_HAS_TLS)
@@ -492,26 +513,17 @@ natsSock_Write(natsSockCtx *ctx, const char *data, int len, int *n)
     {
 #if defined(NATS_HAS_TLS)
         if (ctx->ssl != NULL)
+        {
+            int sslErr = 0;
+
+            natsMutex_Lock(ctx->sslMu);
             bytes = SSL_write(ctx->ssl, data, len);
-        else
-#endif
-#ifdef MSG_NOSIGNAL
-            bytes = send(ctx->fd, data, len, MSG_NOSIGNAL);
-#else
-            bytes = send(ctx->fd, data, len, 0);
-#endif
+            if (bytes < 0)
+                sslErr = SSL_get_error(ctx->ssl, bytes);
+            natsMutex_Unlock(ctx->sslMu);
 
-        if (bytes == 0)
-        {
-            return nats_setDefaultError(NATS_CONNECTION_CLOSED);
-        }
-        else if (bytes < 0)
-        {
-#if defined(NATS_HAS_TLS)
-            if (ctx->ssl != NULL)
+            if (sslErr != 0)
             {
-                int sslErr = SSL_get_error(ctx->ssl, bytes);
-
                 if (sslErr == SSL_ERROR_ZERO_RETURN)
                     return nats_setDefaultError(NATS_CONNECTION_CLOSED);
 
@@ -528,8 +540,21 @@ natsSock_Write(natsSockCtx *ctx, const char *data, int len, int *n)
                     continue;
                 }
             }
+        }
+        else
+#endif
+#ifdef MSG_NOSIGNAL
+            bytes = send(ctx->fd, data, len, MSG_NOSIGNAL);
+#else
+            bytes = send(ctx->fd, data, len, 0);
 #endif
 
+        if (bytes == 0)
+        {
+            return nats_setDefaultError(NATS_CONNECTION_CLOSED);
+        }
+        else if (bytes < 0)
+        {
             if (NATS_SOCK_GET_ERROR != NATS_SOCK_WOULD_BLOCK)
             {
 #if defined(NATS_HAS_TLS)
