@@ -1,4 +1,4 @@
-// Copyright 2015-2023 The NATS Authors
+// Copyright 2015-2025 The NATS Authors
 // Licensed under the Apache License, Version 2.0 (the "License");
 // you may not use this file except in compliance with the License.
 // You may obtain a copy of the License at
@@ -117,6 +117,9 @@ nats_JSONGetInt32(nats_JSON *json, const char *fieldName, int32_t *value);
 natsStatus
 nats_JSONGetUInt16(nats_JSON *json, const char *fieldName, uint16_t *value);
 
+natsStatus
+nats_JSONGetUInt32(nats_JSON *json, const char *fieldName, uint32_t *value);
+
 natsStatus
 nats_JSONGetBool(nats_JSON *json, const char *fieldName, bool *value);
 
@@ -204,6 +207,9 @@ nats_Base32_DecodeString(const char *src, char *dst, int dstMax, int *dstLen);
 natsStatus
 nats_Base64RawURL_EncodeString(const unsigned char *src, int srcLen, char **pDest);
 
+natsStatus
+nats_Base64URL_EncodeString(const unsigned char *src, int srcLen, char **pDest);
+
 natsStatus
 nats_Base64_Encode(const unsigned char *src, int srcLen, char **pDest);
 
@@ -240,17 +246,25 @@ nats_marshalLong(natsBuffer *buf, bool comma, const char *fieldName, int64_t lva
 natsStatus
 nats_marshalULong(natsBuffer *buf, bool comma, const char *fieldName, uint64_t uval);
 
+char *
+nats_formatDuration(char *buf, int64_t nano);
+
+static inline char *nats_formatDurationMilli(char *buf, int64_t milli)
+{
+    return nats_formatDuration(buf, milli * 1000 * 1000);
+}
+
 natsStatus
 nats_marshalDuration(natsBuffer *out_buf, bool comma, const char *field_name, int64_t d);
 
 natsStatus
-nats_marshalMetadata(natsBuffer *buf, bool comma, const char *fieldName, natsMetadata md);
+nats_marshalMetadata(natsBuffer *buf, bool comma, const char *fieldName, natsMetadata *md);
 
 natsStatus
 nats_unmarshalMetadata(nats_JSON *json, const char *fieldName, natsMetadata *md);
 
 natsStatus
-nats_cloneMetadata(natsMetadata *clone, natsMetadata md);
+nats_cloneMetadata(natsMetadata *clone, natsMetadata *md);
 
 void
 nats_freeMetadata(natsMetadata *md);
@@ -264,4 +278,25 @@ nats_parseTime(char *str, int64_t *timeUTC);
 natsStatus
 nats_formatStringArray(char **out, const char **strings, int count);
 
+natsStatus
+nats_marshalStringArray(natsBuffer *buf, bool comma, const char *fieldName, const char **values, int len);
+
+natsStatus
+nats_marshalTimeUTC(natsBuffer *buf, bool comma, const char *fieldName, int64_t timeUTC);
+
+natsStatus
+nats_marshalString(natsBuffer *buf, bool omitEmpty, bool comma, const char *fieldName, const char *str);
+
+natsStatus
+nats_validateLimitedTerm(const char *name, const char *term);
+
+bool
+nats_validBucketName(const char *bucket);
+
+natsStatus
+nats_unmarshalHeader(nats_JSON *json, const char *fieldName, natsHeader **header);
+
+natsStatus
+nats_marshalHeader(natsBuffer *buf, bool omitEmpty, bool comma, const char *fieldName, natsHeader *header);
+
 #endif /* UTIL_H_ */
